/*
 * UHUPolyCanvas.java
 *
 * Created on 26 de noviembre de 2007, 11:02
 */

package interfaz;

import java.awt.Color;
import java.awt.Image;
import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.image.*;
import java.beans.*;
import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JPanel;

/**
 * @author Pandre
 */
public class UHUPolyCanvas extends JPanel implements Serializable {
    
    private Image imgCasillas[] = new Image[40];
    private Image imgJugadores[] = new Image[6];
    private Image imgCasas[] = new Image[4];
    private Image imgHoteles[] = new Image[4];
    private Image imgPestanasPropiedad[] = new Image[7];

    private Image dbDibujo = null;  // imagen para el doble bfer
    private Graphics dbDibujo_g;    // contexto grfico para el doble bfer

    public void inicializar() {
        
        for (int i = 0; i < 40; i++)
            imgCasillas[i] = new ImageIcon("res/casillag" + (i+1) + ".png").getImage();
        
        for (int i = 0; i < 6; i++)
            imgJugadores[i] = new ImageIcon("res/jugador" + (i+1) + ".png").getImage();
        
        for (int i = 0; i < 4; i++)
            imgCasas[i] = new ImageIcon("res/casa" + (i+1) + ".png").getImage();
        
        for (int i = 0; i < 4; i++)
            imgHoteles[i] = new ImageIcon("res/hotel" + (i+1) + ".png").getImage();
        
        for (int i = 0; i < 7; i++)
            imgPestanasPropiedad[i] = new ImageIcon("res/pestanaPropiedad" + (i) + ".png").getImage();
        
        if (dbDibujo == null)
            dbDibujo = createImage(getWidth(), getHeight());

        
        dbDibujo_g = dbDibujo.getGraphics();
        dbDibujo_g.setColor(Color.white);
        dbDibujo_g.fillRect(0, 0, getWidth(), getHeight());

        for (int i = 1; i <= 40; i++)
            pintarCasilla(i);
        
        pintaLogo(false);
    
    }

    public void update(Graphics g) {
        // redefinimos para que slo pinte en paintComponent y no borre todo.
        paintComponent(g);
    }

    public void paintComponent(Graphics g) {
        if (dbDibujo != null)
            g.drawImage(dbDibujo, 0, 0, this);  // copia la imagen del doble bfer en el componente
    }
    
    private Posicion obtenerPos(int numCasilla) {
//        int posiciones[][] = {{630,630},{570,630},{510,630},{450,630},{390,630},
//                {330,630},{270,630},{210,630},{150,630},{90,630}, {0,630},
//                {0,570},{0,510},{0,450},{0,390},{0,330},{0,270},{0,210},{0,150},
//                {0,90},{0,0},{90,0},{150,0},{210,0},{270,0},{330,0},{390,0},
//                {450,0},{510,0},{570,0},{630,0},{630,90},{630,150},{630,210},
//                {630,270},{630,330},{630,390},{630,450},{630,510},{630,570}};
        int posiciones[][] = {{798,798},{722,798},{646,798},{570,798},{494,798},
                {418,798},{342,798},{266,798},{190,798},{114,798}, {0,798},
                {0,722},{0,646},{0,570},{0,494},{0,418},{0,342},{0,266},{0,190},
                {0,114},{0,0},{114,0},{190,0},{266,0},{342,0},{418,0},{494,0},
                {570,0},{646,0},{722,0},{798,0},{798,114},{798,190},{798,266},
                {798,342},{798,418},{798,494},{798,570},{798,646},{798,722}};
        
        int orient;     
        if ((numCasilla >=2) && (numCasilla <= 11)) orient = 0;
        else if ((numCasilla >=12) && (numCasilla <= 20)) orient = 1;
        else if ((numCasilla >=21) && (numCasilla <= 30)) orient = 2;
        else orient = 3;
        Posicion res = new Posicion(posiciones[numCasilla-1][0], posiciones[numCasilla-1][1], orient);
        return res;
    }
    
    public void pintaLogo(boolean aclarado) {
    
        Image logo;
        if (aclarado)
            logo = new ImageIcon("res/logotipoablend.png").getImage();
        else
            logo = new ImageIcon("res/logotipo.png").getImage();
        
        dbDibujo_g = dbDibujo.getGraphics();

        dbDibujo_g.drawImage(logo, 233, 340, Color.white, this);

        repaint(); 
    
    }

    public void pintarCasilla(int numCasilla) {

        Posicion pos = obtenerPos(numCasilla);
        dbDibujo_g = dbDibujo.getGraphics();

        dbDibujo_g.drawImage(imgCasillas[numCasilla-1], pos.getX(), pos.getY(), Color.white, this);

        repaint();  
        
    }
    
    
    public void pintarJugador(int jugador, int numCasilla) {
            // [orient][x][y]
        int desp[][][] = {{{10,27},{41,27},{10,55},{41,55},{10,83},{41,83}}, // abajo
                         {{4,10},{32,10},{60,10},  {4,40},{32,40},{60,40}},  // izq
                         {{10,9},{40,9},{10,36},{40,36},{10,60},{40,60}},  // arriba
                         {{27,10},{55,10},{83,10},  {27,40},{55,40},{83,40}}}; // der
        
        Posicion pos = obtenerPos(numCasilla);
        dbDibujo_g = dbDibujo.getGraphics();

        dbDibujo_g.drawImage(imgJugadores[jugador-1], pos.getX()+desp[pos.getOrient()][jugador-1][0],
                             pos.getY()+desp[pos.getOrient()][jugador-1][1], this);

        repaint();  
        
    }
    
    public void pintarCasa(int numCasas, int numCasilla) {
            // [orient][x][y]
        int desp[][][] = {{{2,3},{20,3},{38,3},{56,3}}, // abajo
                         {{93,2},{93,20},{93,38},{93,56}},  // izq
                         {{2,93},{20,93},{38,93},{56,93}},  // arriba
                         {{3,2},{3,20},{3,38},{3,56}}}; // der
        
        Posicion pos = obtenerPos(numCasilla);
        dbDibujo_g = dbDibujo.getGraphics();

        for (int i = 0; i < numCasas; i++)
            dbDibujo_g.drawImage(imgCasas[pos.getOrient()], pos.getX()+desp[pos.getOrient()][i][0],
                             pos.getY()+desp[pos.getOrient()][i][1], this);

        repaint();  
        
    }
    
    public void pintarHotel(int numCasilla) {
            // [orient][x][y]
        int desp[][][] = {{{2,3}}, // abajo
                          {{93,2}},  // izq
                          {{2,93}},  // arriba
                          {{3,2}}}; // der
        
        Posicion pos = obtenerPos(numCasilla);
        dbDibujo_g = dbDibujo.getGraphics();

        dbDibujo_g.drawImage(imgHoteles[pos.getOrient()], pos.getX()+desp[pos.getOrient()][0][0],
                             pos.getY()+desp[pos.getOrient()][0][1], this);

        repaint();  
        
    }
    
    public void pintarPropiedad(int numCasilla, int numJugador) {
            // [orient][x][y]
        int desp[][][] = {{{-8,-51}}, // abajo
                          {{75,-8}},  // izq
                          {{-7,75}},  // arriba
                          {{-51,-6}}}; // der
        
        Posicion pos = obtenerPos(numCasilla);
        dbDibujo_g = dbDibujo.getGraphics();

        dbDibujo_g.drawImage(rotarImagen(imgPestanasPropiedad[numJugador], pos.getOrient()*90-90).getImage(), pos.getX()+desp[pos.getOrient()][0][0],
                             pos.getY()+desp[pos.getOrient()][0][1], this);

        repaint();  
        
    }
    
    /*
    * Funcin tomada y adaptada desde el ejemplo mostrado en:
    *   http://saloon.javaranch.com/cgi-bin/ubb/ultimatebb.cgi?ubb=get_topic&f=2&t=013741
    */
    private ImageIcon rotarImagen(Image img, int grados)
    {
        // Carga la imagen
        BufferedImage imagen = null;

        imagen = toBufferedImage(img);
        
        // Imagen resultante
        ImageIcon icono = new ImageIcon();
        {
            // Creamos una imagen temporal en memoria
            BufferedImage bi = new BufferedImage(90, 90, BufferedImage.TYPE_INT_ARGB);
            Graphics2D g2 = bi.createGraphics();

            // Definimos la transformacin
            double angulo = Math.toRadians(grados);
            double x = (90 - imagen.getWidth())/2;
            double y = (90 - imagen.getHeight())/2;
            AffineTransform at = AffineTransform.getTranslateInstance(x,y);
            at.rotate(angulo, imagen.getWidth()/2, imagen.getHeight()/2);
            
            // Dibujamos la imagen original con la transformacin en la temporal
            g2.drawRenderedImage(imagen, at);
            
            // Liberamos la memoria usada
            g2.dispose();
            
            // Devolvemos la imagen rotada
            icono = new ImageIcon(bi);
        }

        return icono;
    }
    
    /*
    * Funcin tomada y adaptada desde el ejemplo mostrado en:
    *   http://saloon.javaranch.com/cgi-bin/ubb/ultimatebb.cgi?ubb=get_topic&f=2&t=013741
    */
    public static ImageIcon rotarImagenCasilla(int numCasilla)
    {
        
        int grados;
       // int numCasilla = casilla.getIdCasilla();
        if ((numCasilla > 1) && (numCasilla < 11)) grados = 0;
        else if ((numCasilla > 11) && (numCasilla < 21)) grados = -90;
        else if ((numCasilla > 21) && (numCasilla < 31)) grados = 180;
        else grados = 90;
        if ((numCasilla == 26) || (numCasilla == 29)) grados = 0;
        
        // Carga la imagen
        BufferedImage imagen = null;
        try
        {
            imagen = ImageIO.read(new File("res\\casilla" + numCasilla + ".png"));
        }
        catch(IOException ioe) {}
        
        // Imagen resultante
        ImageIcon icono = new ImageIcon();
        {
            // Creamos una imagen temporal en memoria
            BufferedImage bi = new BufferedImage(90, 90, BufferedImage.TYPE_INT_ARGB);
            Graphics2D g2 = bi.createGraphics();

            // Definimos la transformacin
            double angulo = Math.toRadians(grados);
            double x = (90 - imagen.getWidth())/2;
            double y = (90 - imagen.getHeight())/2;
            AffineTransform at = AffineTransform.getTranslateInstance(x,y);
            at.rotate(angulo, imagen.getWidth()/2, imagen.getHeight()/2);
            
            // Dibujamos la imagen original con la transformacin en la temporal
            g2.drawRenderedImage(imagen, at);
            
            // Liberamos la memoria usada
            g2.dispose();
            
            // Devolvemos la imagen rotada
            icono = new ImageIcon(bi);
        }

        return icono;
    }
    
    /*
    * Funcin tomada y adaptada desde el ejemplo mostrado en:
    *   http://www.exampledepot.com/egs/java.awt.image/Image2Buf.html
    */
    public static BufferedImage toBufferedImage(Image image) {
        if (image instanceof BufferedImage) {
            return (BufferedImage)image;
        }
    
        // This code ensures that all the pixels in the image are loaded
        image = new ImageIcon(image).getImage();
    
        // Determine if the image has transparent pixels; for this method's
        // implementation, see e661 Determining If an Image Has Transparent Pixels
       // boolean hasAlpha = hasAlpha(image);
    
        // Create a buffered image with a format that's compatible with the screen
        BufferedImage bimage = null;
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        try {
            // Determine the type of transparency of the new buffered image
            int transparency = Transparency.OPAQUE;
          //  if (hasAlpha) {
                transparency = Transparency.BITMASK;
            //}
    
            // Create the buffered image
            GraphicsDevice gs = ge.getDefaultScreenDevice();
            GraphicsConfiguration gc = gs.getDefaultConfiguration();
            bimage = gc.createCompatibleImage(
                image.getWidth(null), image.getHeight(null), transparency);
        } catch (HeadlessException e) {
            // The system does not have a screen
        }
    
        if (bimage == null) {
            // Create a buffered image using the default color model
            int type = BufferedImage.TYPE_INT_RGB;
          //  if (hasAlpha) {
                type = BufferedImage.TYPE_INT_ARGB;
            //}
            bimage = new BufferedImage(image.getWidth(null), image.getHeight(null), type);
        }
    
        // Copy image to buffered image
        Graphics g = bimage.createGraphics();
    
        // Paint the image onto the buffered image
        g.drawImage(image, 0, 0, null);
        g.dispose();
    
        return bimage;
    }

    
    class Posicion {
        private int xx;
        private int yy;
        private int orientacion;
        Posicion( int x, int y , int o){
            this.xx = x;
            this.yy = y;
            this.orientacion = o;
        }
        public int getX() { return xx; }
        public int getY() { return yy; }
        public int getOrient() { return orientacion; }
    }
    
}