/*
 * - Cuando no tenga dinero, actualizar la posicion actual
 */

package uhupoly;

import interfaz.*;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;
import javax.persistence.CascadeType;
import javax.persistence.OrderBy;
import javax.persistence.PostLoad;
import javax.persistence.PrePersist;
import javax.persistence.PreUpdate;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;
import javax.swing.JOptionPane;

import java.io.Serializable;
import java.util.List;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;

@Entity
public class Juego implements Serializable {
    
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name="IDJuego")
    private Long id;
    
    private int numJugadores;

    private int turno;
    private int valorDado1;
    private int valorDado2;
    
    @Temporal(value = TemporalType.TIMESTAMP)
    private Calendar fechaGuardado;

    @OneToMany(cascade=CascadeType.ALL)
    private List<TarjetaComunidad> tarjetasComunidad;

    @Transient
    private ArrayList<Dado> dados;

    @OneToMany(cascade={CascadeType.ALL})
    private List<Jugador> jugadores;

    @OneToOne(cascade=CascadeType.ALL)
    private Tablero tablero;

    @OneToMany(cascade=CascadeType.ALL)
    private List<TarjetaSuerte> tarjetasSuerte;
    
    @Transient
    private ObservadorTablero observador;
    
    //------ Atributos para control del transcurso de la partida
    
    private int numVecesDobles;
    @OneToOne
    private Jugador jugadorActual;
    @OneToOne
    private Casilla casillaAnterior;
    
    //------
    
    /**
     * ######### PERSISTENCIA #####################
     */
    public Long getId() {
        return this.id;
    }

    public void setId(Long id) {
        this.id = id;
    }
    
    @Override
    public int hashCode() {
        int hash = 0;
        hash += (this.id != null ? this.id.hashCode() : 0);
        return hash;
    }

    @Override
    public boolean equals(Object object) {
        // TODO: Warning - this method won't work in the case the id fields are not set
        if (!(object instanceof Juego)) {
            return false;
        }
        Juego other = (Juego)object;
        if (this.id != other.id && (this.id == null || !this.id.equals(other.id))) return false;
        return true;
    }

    @Override
    public String toString() {
        
        Date hora = fechaGuardado.getTime();
        DateFormat df = DateFormat.getDateTimeInstance();
        String horaActual = df.format(hora);
        
        String strJugadores = "";
        for (int i = 0; i < numJugadores-1; i++)
            strJugadores = strJugadores + jugadores.get(i).getNombre() + ", ";
        strJugadores = strJugadores + jugadores.get(numJugadores-1).getNombre();
        
        
        String res = horaActual + "  -  Jugadores: " + strJugadores;
        return res;
    }
    
     /**
     * ######### FIN PERSISTENCIA #####################
     */
    

    public Juego() {
        // creamos los dados
        dados = new ArrayList<Dado>(); 
        dados.add(new Dado());
        dados.add(new Dado());
        
        // creamos el tablero
        tablero = new Tablero();
        
        // creamos los jugadores
        jugadores = new ArrayList<Jugador>();
        
        // creamos las tarjetas de comunidad
        tarjetasComunidad = new ArrayList<TarjetaComunidad>();
        tarjetasComunidad.add(new TarjetaComunidad("Sitese en la Avenida de Andaluca.", 0));
        tarjetasComunidad.add(new TarjetaComunidad("Pague la factura del mdico: 30 ", 1));
        tarjetasComunidad.add(new TarjetaComunidad("Recibir su inters sobre el prestamo al 7%: 15 ", 2));
        tarjetasComunidad.add(new TarjetaComunidad("En su aniversario reciba de cada jugador 6 .", 3));
        tarjetasComunidad.add(new TarjetaComunidad("Pague al hospital: 60 .", 4));
        tarjetasComunidad.add(new TarjetaComunidad("Vaya directamente a la Crcel sin pasar por la casilla de salida.", 5));
        tarjetasComunidad.add(new TarjetaComunidad("Pague una multa de 6 .", 6));
        tarjetasComunidad.add(new TarjetaComunidad("Cobre una herencia de 60 .", 7));
        tarjetasComunidad.add(new TarjetaComunidad("Ha ganado el segundo premio de belleza: 6 ", 8));
        tarjetasComunidad.add(new TarjetaComunidad("Pague su poliza de seguros: 30 ", 9));
        tarjetasComunidad.add(new TarjetaComunidad("Contribuciones le devuelven 12 ", 10));
        tarjetasComunidad.add(new TarjetaComunidad("La venta de su stock le producen 30 ", 11));
        tarjetasComunidad.add(new TarjetaComunidad("Coloquese en la casilla de Salida.", 12));
        tarjetasComunidad.add(new TarjetaComunidad("Error de la banca a su favor. Reciba 238 ", 13));
        tarjetasComunidad.add(new TarjetaComunidad("Queda libre de la carcel. Esta carta puede conservarse o venderse a otros jugadores.", 14));
        tarjetasComunidad.add(new TarjetaComunidad("Retroceda tres casillas.", 15));
        
        // creamos las tarjetas de suerte
        tarjetasSuerte = new ArrayList<TarjetaSuerte>();
        tarjetasSuerte.add(new TarjetaSuerte("Vaya a la calle de Bilbao. Si pasa por la casilla de SALIDA, cobre 238 ", 0));
        tarjetasSuerte.add(new TarjetaSuerte("La inspeccin de la calle le obliga a reparaciones. Pague: 24  por casa y 66  por hotel.", 1));
        tarjetasSuerte.add(new TarjetaSuerte("Pague por gastos escolares 90 .", 2));
        tarjetasSuerte.add(new TarjetaSuerte("Vaya a la estacin de Autobuses. Si pasa por la casilla de SALIDA, cobre 238 .", 3));
        tarjetasSuerte.add(new TarjetaSuerte("Adelante hasta la casilla de SALIDA.", 4));
        tarjetasSuerte.add(new TarjetaSuerte("Multa por embriaguez: 12 ", 5));
        tarjetasSuerte.add(new TarjetaSuerte("Vaya a la calle de Coln. Si pasa por la casilla de SALIDA, cobre 238 .", 6));
        tarjetasSuerte.add(new TarjetaSuerte("Vaya a G.V. Fernando el Catlico. Si pasa por la casilla de SALIDA, cobre 238 .", 7));
        tarjetasSuerte.add(new TarjetaSuerte("Multa por exceso de velocidad: 9 .", 8));
        tarjetasSuerte.add(new TarjetaSuerte("La Banca arroja un dividendo de 30 .", 9));
        tarjetasSuerte.add(new TarjetaSuerte("Vaya directamente a la Crcel sin pasar por la casilla de salida.", 10));
        tarjetasSuerte.add(new TarjetaSuerte("Su inmueble y su prstamo producen 90 .", 11));
        tarjetasSuerte.add(new TarjetaSuerte("Retroceda tres casillas.", 12));
        tarjetasSuerte.add(new TarjetaSuerte("Ha ganado el premio de palabras cruzadas: 60 ", 13));
        tarjetasSuerte.add(new TarjetaSuerte("Queda libre de la carcel. Esta carta puede conservarse o venderse a otros jugadores.", 14));
        tarjetasSuerte.add(new TarjetaSuerte("Haga reparaciones en todas sus casas. Pague por cada casa 15  y por cada hotel 60 .", 15));

        setTurno(0);
        
        fechaGuardado = new GregorianCalendar().getInstance();
        
    }

    
    public List<TarjetaComunidad> getTarjetasComunidad() {
        return tarjetasComunidad;
    }

    public void setTarjetasComunidad(ArrayList<TarjetaComunidad> val) {
        this.tarjetasComunidad = val;
    }

    public ArrayList<Dado> getDados() {
        return dados;
    }

    public void setDados(ArrayList<Dado> val) {
        this.dados = val;
    }

    
    public List<Jugador> getJugadores() {
        return jugadores;
    }

    public void setJugadores(ArrayList<Jugador> val) {
        this.jugadores = val;
    }

    
    public Tablero getTablero() {
        return tablero;
    }

    public void setTablero(Tablero val) {
        this.tablero = val;
    }


    public List<TarjetaSuerte> getTarjetasSuerte() {
        return tarjetasSuerte;
    }

    public void setTarjetasSuerte(ArrayList<TarjetaSuerte> val) {
        this.tarjetasSuerte = val;
    }

    public int getNumJugadores() {
        return numJugadores;
    }

    public void setNumJugadores(int val) {
        this.numJugadores = val;
    }
    
    public int getValorDado1() {
        return valorDado1;
    }

    public void setValorDado1(int val) {
        this.valorDado1 = val;
    }
    
    public int getValorDado2() {
        return valorDado2;
    }

    public void setValorDado2(int val) {
        this.valorDado2 = val;
    }
    
    public ObservadorTablero getObservador() {
        return observador;
    }

    public void setObservador(ObservadorTablero val) {
        this.observador = val;
    }

    public void tirarDados() {
        dados.get(0).lanzarDado();
        valorDado1 = dados.get(0).getValorCara();
        dados.get(1).lanzarDado();
        valorDado2 = dados.get(1).getValorCara();
        
        observador.muestraTirada(valorDado1, valorDado2);
    }

    public int getTurno() {
        return turno;
    }

    public void setTurno(int val) {
        this.turno = val;
    }
    
    @SuppressWarnings("unchecked")
    public void cargadoDesdeBBDD() {
    
        Collections.sort(tablero.getCasillas());
        Collections.sort(tarjetasComunidad);
        Collections.sort(tarjetasSuerte);
        Collections.sort(jugadores);
        
        observador.muestraJugadores(this);
        for (int i = 0; i < 40; i++)
            observador.actualizaCasilla(tablero.getCasillas().get(i));
        
        Jugador jugador;
        
        for (int i = 0; i < numJugadores; i++)
        {
            jugador = jugadores.get(i);
            observador.actualizaJugador(jugador);
            if (jugador.getArruinado())
                observador.muestraArruinado(jugador);
            if ((jugador.getPosicion().getIdCasilla() == 11) && (jugador.getDeVisita() == false))
                observador.muestraEncarcelado(jugador, true);
        }
        
        observador.muestraMensajeSecundario("");
        
        if ((jugadorActual.getPosicion().getIdCasilla() == 11) && (jugadorActual.getDeVisita() == false))
            observador.muestraMensajeSecundario("Recuerda que ests en la crcel!");
        
        observador.mostrarLogotipo(true);
        
        observador.muestraTurnoLanzar(jugadorActual.getNombre());
    
    }
    
    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     */
    
    public void comenzarJuego() {
        turno = 0;
        numVecesDobles = 0;
        jugadorActual = jugadores.get(getTurno());
        observador.muestraTurnoLanzar(jugadores.get(turno).getNombre());
        observador.mostrarLogotipo(true);
        
    }
    
    public void cambiarTurno() {
        
        Jugador jugadorAnterior = jugadorActual;
        int numArruinados = 0;
        
        do {
            turno = (turno + 1) % numJugadores;
            numVecesDobles = 0;
            jugadorActual = jugadores.get(getTurno());
        } while (jugadorActual.getArruinado());
        
        for (int i = 0; i < numJugadores; i++)
            if (jugadores.get(i).getArruinado())
                numArruinados++;
        
        if (numJugadores ==  (numArruinados+1))
        {
            // Mostrar fin del juego con el ganador jugadorActual.
            observador.muestraFinDelJuego(this, jugadorActual);
            return;
        }
        
        if ((jugadorActual.getPosicion().getIdCasilla() == 11) && (jugadorActual.getDeVisita() == false))
            observador.muestraMensajeSecundario("Recuerda que ests en la crcel!");
        
        observador.muestraTurnoLanzar(jugadores.get(turno).getNombre());

    }
    
    public void jugar() {
    
        boolean dobles;
        Casilla casillaActual;
        Casilla casillaDestino;

        // ver si esta en la carcel para poder usar las tarjetas de salida
        if ((jugadorActual.getPosicion().getIdCasilla() == 11) && (jugadorActual.getDeVisita() == false) &&
                (jugadorActual.getTarjetaJugador().size() > 0))
        {
            
            if (JOptionPane.showConfirmDialog(null, "TE ENCUENTRAS EN LA CARCEL.\nTienes la tarjeta para salir gratis de la crcel.\nQuieres usarla?",
                                    "Salir de la crcel", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
            {
                
                Tarjeta tarjeta = jugadorActual.getTarjetaJugador().get(0);
                jugadorActual.getTarjetaJugador().remove(tarjeta);
                tarjeta.setPropietario(null);
                
                jugadorActual.setTurnosEnCarcel(0);
                jugadorActual.setDeVisita(true);
                    //muestra que queda libre por dobles
                observador.muestraInformacion("Quedas libre de la crcel por usar tarjeta!!!");
                observador.muestraEncarcelado(jugadorActual, false);
                    //muestra tirar dados
                observador.muestraTurnoLanzar(jugadorActual.getNombre());
                return;
            
            }
            
        }
 
        
        tirarDados();
        
        dobles = (valorDado1 == valorDado2);
        //***** notifica tirada
        observador.muestraTirada(valorDado1, valorDado2);
        
        // si esta en la carcel
        if ((jugadorActual.getPosicion().getIdCasilla() == 11) && (jugadorActual.getDeVisita() == false))
        {
        
            observador.muestraMensajeSecundario("Recuerda que ests en la crcel!");
            
            
            if (jugadorActual.getTurnosEnCarcel() > 1) // le quedan ms de un turno
            {
                
                if (dobles)
                {
			//se saca de la carcel
                    jugadorActual.setTurnosEnCarcel(0);
                    jugadorActual.setDeVisita(true);
			//muestra que queda libre por dobles
                    observador.muestraInformacion("Quedas libre de la crcel por sacar dobles!!!");
                    observador.muestraEncarcelado(jugadorActual, false);
			//muestra tirar dados
                    observador.muestraTurnoLanzar(jugadorActual.getNombre());
                    return;
                } else
                {
                    
                    boolean quierePagar = false;
                    
                    if (jugadorActual.tieneEfectivo(60f))
                        quierePagar = (JOptionPane.showConfirmDialog(null, "Deseas pagar 60 y salir directamente de la crcel?",
                                    "Salir de la crcel", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION);
                    else
                        observador.muestraInformacion("No tienes dinero para pagar la fianza... Sigues otro turno.");
                    
                    if (quierePagar)
                    {
			//se saca de la carcel
                        jugadorActual.setTurnosEnCarcel(0);
                        jugadorActual.setDeVisita(true);
                        
                        
                        // se le cobran los 60
                        jugadorActual.setCapital(jugadorActual.getCapital() - 60f);
                        observador.actualizaJugador(jugadorActual);
                        
			//muestra que queda libre
                        observador.muestraInformacion("Quedas libre de la crcel por pagar fianza");
                        observador.muestraEncarcelado(jugadorActual, false);
			//muestra tirar dados
                        observador.muestraTurnoLanzar(jugadorActual.getNombre());
                        return;
                    } else
                    {
                        //decrementa turnos en la carcel
                        jugadorActual.setTurnosEnCarcel(jugadorActual.getTurnosEnCarcel()-1);
			//activa TERMINAR____________
                        observador.muestraTerminarTurno();
                        return;
                    }
                }
                
            } else // ste es el ltimo turno que est
            {
                    
                if (dobles)
                {
                    //muestra que queda libre por dobles
                    observador.muestraInformacion("Quedas libre de la crcel por sacar dobles!!!</b><br>Vuelves a tirar otra vez</html>");
                    observador.muestraEncarcelado(jugadorActual, false);
                    jugadorActual.setTurnosEnCarcel(0);
                    jugadorActual.setDeVisita(true);
                    observador.muestraTurnoLanzar(jugadorActual.getNombre());
                    return;
                } else
                {
                    //pagua 10.000
                    // comprobar si tiene suficiente en dinero. De no ser as,
                    // mostrar aqu la obligacin de tener que hipotecar.
                    while (!jugadorActual.tieneEfectivo(60f))
                    {

                        if (jugadorActual.estaArruinado(60f))
                        {
                            jugadorActual.entregarTodoA(null, this);
                            observador.muestraArruinado(jugadorActual);
                            observador.muestraTerminarTurno();
                            return;
                        }
                        else observador.muestraSinDinero(jugadorActual, 60f);
                    }
                
                    jugadorActual.setCapital(jugadorActual.getCapital() - 60f);
                    //muestra que queda libre por pagar
                    observador.muestraInformacion("<b>Quedas libre de la crcel y pagas al salir</b>");
                    observador.muestraEncarcelado(jugadorActual, false);
                }
                //se saca de la carcel
                jugadorActual.setTurnosEnCarcel(0);
                jugadorActual.setDeVisita(true);
                //muestra tirar dados
                observador.muestraTurnoLanzar(jugadorActual.getNombre());
            }
            
            
        } // fin de si est en la crcel
        
        if (dobles)
            numVecesDobles += 1;
        
        if (numVecesDobles == 3)
        {
            //directo a la carcel
            jugadorActual.setTurnosEnCarcel(3);
            jugadorActual.setDeVisita(false);
            casillaActual = jugadorActual.getPosicion();
            jugadorActual.setPosicion(tablero.getCasillas().get(10));
            //muestra que son 3 dobles y va a la carcel
            observador.muestraMensaje("<b>Vas a la crcel por sacar 3 veces dobles!!!</b>");
            
            observador.muestraEncarcelado(jugadorActual, true);
            
            observador.actualizaCasilla(casillaActual);
            observador.actualizaCasilla(tablero.getCasillas().get(10));
            
            //activa TERMINAR_____________
            observador.inactivaOpciones();
            observador.muestraTerminarTurno();
            return;
        }
        
        //muestra resultado de tirada en pantalla, se hace lo que haya.
        casillaActual = jugadorActual.getPosicion();
        casillaDestino = tablero.getCasillas().get((casillaActual.getIdCasilla()-1 + valorDado1 + valorDado2) % 40);
        
        // Si pasa por la casilla de salida, cobrar dinero
        if (casillaDestino.getIdCasilla() < casillaActual.getIdCasilla())
        {
            jugadorActual.setCapital(jugadorActual.getCapital() + 238f);
            observador.actualizaJugador(jugadorActual);
        }
        
        // se hace lo que haya en esa casilla
        boolean seguirTurno = casillaDestino.accion(jugadorActual, this);
        
        // actualizar en la interfaz grfica
        observador.actualizaJugador(jugadorActual);
        observador.actualizaCasilla(casillaActual);
        observador.actualizaCasilla(casillaDestino);
        
        // Si por la accin no debe seguir con su turno, terminarlo.
        if (!seguirTurno) return;
        
	//activa opciones
        observador.activaOpciones();
        
        //si no saco dobles, activa TERMINAR
        if (!dobles)
        {
        
            observador.muestraTerminarTurno();
            
        } else //si saco dobles:
        {
        
            //turno al mismo, muestra quien tiene el turno (por dobles), muestra tirar dados
            observador.muestraTurnoLanzar(jugadores.get(turno).getNombre());
        
        }
        
    
    }

    public void resetJuego() {
    
        Solar solar;

        jugadores.clear();
        for (int i = 0; i < 40; i++)
        {
            if (tablero.getCasillas().get(i) instanceof Solar)
            {
                solar = ((Solar)tablero.getCasillas().get(i));
                solar.setPropietario(null);
                solar.setNumCasas(0);
                solar.setTieneHotel(false);
            }
            if (tablero.getCasillas().get(i) instanceof Estacion)
                ((Estacion)tablero.getCasillas().get(i)).setPropietario(null);
            if (tablero.getCasillas().get(i) instanceof ServicioPublico)
                ((ServicioPublico)tablero.getCasillas().get(i)).setPropietario(null);
        }
        
        
        tarjetasComunidad.get(14).setPropietario(null);
        tarjetasSuerte.get(14).setPropietario(null);    
    
    }

    public Calendar getFechaGuardado() {
        return fechaGuardado;
    }

    public void setFechaGuardado(Calendar fechaGuardado) {
        this.fechaGuardado = fechaGuardado;
    }

}
