package uhupoly;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;

@Entity
public class Jugador implements Serializable, Comparable {

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name="IDJugador")
    private Long id;

    public Jugador() {
        nombre = "";
        capital = 1800f;
        deVisita = true;
        turnosEnCarcel = 0;
        posicion = null;
        tarjetaJugador = null;
        propiedades = new ArrayList<Propiedad>();
        tarjetaJugador =  new ArrayList<Tarjeta>();
        arruinado = false;
    }
    private String nombre;

    private float capital;

    private boolean deVisita;

    private int turnosEnCarcel;

    @OneToOne
    private Casilla posicion;

    @OneToMany(cascade=CascadeType.ALL)
    private List<Tarjeta> tarjetaJugador;

    @OneToMany(cascade=CascadeType.ALL)
    private List<Propiedad> propiedades;
    
    private int numJugador;
    
    private boolean arruinado;
    

    public Jugador(String nom) {
        
        nombre = nom;
        capital = 1800f;
        deVisita = true;
        turnosEnCarcel = 0;
        posicion = null;
        tarjetaJugador = null;
        propiedades = new ArrayList<Propiedad>();
        tarjetaJugador =  new ArrayList<Tarjeta>();
        arruinado = false;
        
    }
    
    public int compareTo(Object o) {
        return (numJugador - ((Jugador)o).numJugador);
    }
    
     /**
     * ######### PERSISTENCIA #####################
     */
    public Long getId() {
        return this.id;
    }

    public void setId(Long id) {
        this.id = id;
    }
    
    @Override
    public int hashCode() {
        int hash = 0;
        hash += (this.id != null ? this.id.hashCode() : 0);
        return hash;
    }

    @Override
    public boolean equals(Object object) {
        // TODO: Warning - this method won't work in the case the id fields are not set
        if (!(object instanceof Jugador)) {
            return false;
        }
        Jugador other = (Jugador)object;
        if (this.id != other.id && (this.id == null || !this.id.equals(other.id))) return false;
        return true;
    }

    @Override
    public String toString() {
        String res = "Jugador(" + nombre + ")";
        return res;
    }
    
     /**
     * ######### FIN PERSISTENCIA #####################
     */

    public String getNombre() {
        return nombre;
    }

    public void setNombre(String val) {
        this.nombre = val;
    }

    public float getCapital() {
        return capital;
    }

    public void setCapital(float val) {
        this.capital = val;
    }

    public boolean getDeVisita() {
        return deVisita;
    }

    public void setDeVisita(boolean val) {
        this.deVisita = val;
    }

    public int getTurnosEnCarcel() {
        return turnosEnCarcel;
    }

    public void setTurnosEnCarcel(int val) {
        this.turnosEnCarcel = val;
    }

    public Casilla getPosicion() {
        return posicion;
    }

    public void setPosicion(Casilla val) {
        this.posicion = val;
    }

    
    public List<Tarjeta> getTarjetaJugador() {
        return tarjetaJugador;
    }

    public void setTarjetaJugador(ArrayList<Tarjeta> val) {
        this.tarjetaJugador = val;
    }

    public List<Propiedad> getPropiedades() {
        return propiedades;
    }

    public void setPropiedades(ArrayList<Propiedad> val) {
        this.propiedades = val;
    }
    
    public boolean getArruinado() {
        return arruinado;
    }

    public void setArruinado(boolean val) {
        this.arruinado = val;
    }
    
    public boolean tieneEfectivo(float cantidad) {
        return (cantidad <= capital);
    }
    
    public boolean estaArruinado(float cantidadDebida) {
        boolean res = false;
        
        float sumaBienes = 0;
        Propiedad propiedad;
        Solar solar;
        
        for (int i = 0; i < propiedades.size(); i++)
        {
            propiedad = propiedades.get(i);
            if (!propiedad.getHipotecada())
            {
            
                if (propiedad instanceof Solar)
                {
                    if (((Solar)propiedad).getTieneHotel())
                        sumaBienes += ((Solar)propiedad).getPrecioHotel() / 2;
                    else
                        sumaBienes += ((Solar)propiedad).getNumCasas() * ((Solar)propiedad).getPrecioCasa() / 2;
                }

                sumaBienes += propiedad.getValorHipoteca();

            }
        }
        
        sumaBienes += capital;
        
        res = (sumaBienes < cantidadDebida);
        
        return res;
    }
    
    public void entregarTodoA(Jugador jugador, Juego juego) {
    
        arruinado = true;
        
        if (jugador != null)
        {
        
            float capitalConstrucciones = 0;
            Solar solar;
            for (int i = 0; i < propiedades.size(); i++)
                if (propiedades.get(i) instanceof Solar)
                {
                    solar = ((Solar)propiedades.get(i));
                    if (solar.getTieneHotel())
                        capitalConstrucciones += solar.getPrecioHotel() / 2;
                    else
                        capitalConstrucciones += solar.getPrecioCasa() * solar.getNumCasas() / 2;
                } 

            Propiedad propiedad;
            float interesesHipoteca = 0;
            for (int i = 0; i < propiedades.size(); i++)
            {
                propiedad = propiedades.get(i);
                propiedad.setPropietario(jugador);
                jugador.getPropiedades().add(propiedad);
                juego.getObservador().actualizaCasilla(propiedad);
                if (propiedad.getHipotecada())
                    interesesHipoteca += propiedad.getValorHipoteca() * 0.1f;
            }

            jugador.setCapital(jugador.getCapital() + capitalConstrucciones + this.capital - interesesHipoteca);
            juego.getObservador().actualizaJugador(jugador);
        }
        else
        {
            for (int i = 0; i < propiedades.size(); i++)
            {
                juego.getObservador().muestraOpcionSubasta(propiedades.get(i), juego);
            }
        }
    }
    
    public int getNumJugador() {
        return numJugador;
    }

    public void setNumJugador(int val) {
        this.numJugador = val;
    }

    public Vector<Solar> getSolaresEdificables() {
        
        Vector<Solar> solaresEdificables = new Vector<Solar>();
        
        Solar solar = null;
        Propiedad propiedad = null;
        
        solaresEdificables.clear();
                
        // recorremos todas las propiedades del jugador
        for (int i = 0; i < propiedades.size(); i++)
        {
            propiedad = propiedades.get(i);
            if (propiedad instanceof Solar)
            {
                solar = ((Solar)propiedad);
            
                // comprobar si tambin posee los asociados
                boolean estanTodos = true;
                for (int j = 0; j < solar.getAsociados().size(); j++)
                {
                    if (!propiedades.contains(solar.getAsociados().get(j)))
                        estanTodos = false;
                }
                
                if (estanTodos)
                {
                
                    // obtenemos el n maximo y minimo de casas
                    int maxNumCasas = 0;
                    int minNumCasas = 0;
                    if (solar.getNumCasas() > maxNumCasas)
                            maxNumCasas = solar.getNumCasas();
                    for (int j = 0; j < solar.getAsociados().size(); j++)
                        if (solar.getAsociados().get(j).getNumCasas() > maxNumCasas)
                            maxNumCasas = solar.getAsociados().get(j).getNumCasas();
                    minNumCasas = maxNumCasas;
                    if (solar.getNumCasas() < minNumCasas)
                            minNumCasas = solar.getNumCasas();
                    for (int j = 0; j < solar.getAsociados().size(); j++)
                        if (solar.getAsociados().get(j).getNumCasas() < minNumCasas)
                            minNumCasas = solar.getAsociados().get(j).getNumCasas();
                    
                    // si son iguales los maximos y minimos, mostramos todos los solares del grupo
                    if ((minNumCasas == maxNumCasas) && (!solar.getTieneHotel()) && (!solar.getHipotecada()))
                    {
                        solaresEdificables.add(solar);
                    }
                    // si es menor el minimo, solo mostrar los solares del minimo
                    if ((minNumCasas < maxNumCasas) && (solar.getNumCasas() == minNumCasas) && (!solar.getTieneHotel()) && (!solar.getHipotecada()))
                    {
                        solaresEdificables.add(solar);
                    }
                
                }
                
            }
        }
        
        return solaresEdificables;
    }

    public Vector<Propiedad> getPropiedadesHipotecables() {
        
        Vector<Propiedad> propiedadesHipotecables = new Vector<Propiedad>();
        
        propiedadesHipotecables.clear();
                
        // recorremos todas las propiedades del jugador
        for (int i = 0; i < propiedades.size(); i++)
        {
            if (!((Propiedad)propiedades.get(i)).getHipotecada())
            {
                if (propiedades.get(i) instanceof Solar)
                {
                    Solar solar = ((Solar)propiedades.get(i));
                    if (!solar.getTieneHotel() && solar.getNumCasas() == 0)
                        propiedadesHipotecables.add(propiedades.get(i));
                    
                } else
                    propiedadesHipotecables.add(propiedades.get(i));
            }
        }
        
        return propiedadesHipotecables;
    }
    
    public Vector<Propiedad> getPropiedadesHipotecadas() {
        
        Vector<Propiedad> propiedadesHipotecadas = new Vector<Propiedad>();
        
        propiedadesHipotecadas.clear();
                
        // recorremos todas las propiedades del jugador
        for (int i = 0; i < propiedades.size(); i++)
        {
            if (((Propiedad)propiedades.get(i)).getHipotecada())
                propiedadesHipotecadas.add(propiedades.get(i));
        }
        
        return propiedadesHipotecadas;
    }
    
    public Vector<Propiedad> getPropiedadesVendibles() {
        
        Vector<Propiedad> propiedadesVendibles = new Vector<Propiedad>();
        
        propiedadesVendibles.clear();
                
        Solar solar = null;
        Propiedad propiedad = null;
                
        // recorremos todas las propiedades del jugador
        for (int i = 0; i < propiedades.size(); i++)
        {
            propiedad = propiedades.get(i);
            if (propiedad instanceof Solar)
            {
                solar = ((Solar)propiedad);
            
                // comprobar si tambin posee los asociados
                boolean estanTodos = true;
                for (int j = 0; j < solar.getAsociados().size(); j++)
                {
                    if (!propiedades.contains(solar.getAsociados().get(j)))
                        estanTodos = false;
                }
                
                if (estanTodos)
                {
                
                    // vemos si algun solar esta edificado
                    boolean algunoTieneCasa = false;
                    
                    if ((solar.getNumCasas() > 0) || (solar.getTieneHotel()))
                            algunoTieneCasa = true;
                    
                    for (int j = 0; j < solar.getAsociados().size(); j++)
                        if ((solar.getAsociados().get(j).getNumCasas() > 0) ||
                            (solar.getAsociados().get(j).getTieneHotel()))
                                algunoTieneCasa = true;
                    
                    
                    if (!algunoTieneCasa)
                    {
                        propiedadesVendibles.add(solar);
                    }
                
                }
                else
                    propiedadesVendibles.add(solar);
                
            } else
                propiedadesVendibles.add(propiedad);
        }
        
        return propiedadesVendibles;
    }
    
    public Vector<Solar> getSolaresConstruidos() {
        
        Vector<Solar> solaresConstruidos = new Vector<Solar>();
        
        solaresConstruidos.clear();
                
        Solar solar = null;
        Propiedad propiedad = null;
                
        // recorremos todas las propiedades del jugador
        for (int i = 0; i < propiedades.size(); i++)
        {
            propiedad = propiedades.get(i);
            if (propiedad instanceof Solar)
            {
                solar = ((Solar)propiedad);

                    // obtenemos el n maximo y minimo de casas
                    int maxNumCasas = 0;
                    int minNumCasas = 0;
                    boolean algunoTieneHotel = false;
                    
                    if (solar.getNumCasas() > maxNumCasas)
                            maxNumCasas = solar.getNumCasas();
                    
                    for (int j = 0; j < solar.getAsociados().size(); j++)
                    {
                        if (solar.getAsociados().get(j).getNumCasas() > maxNumCasas)
                            maxNumCasas = solar.getAsociados().get(j).getNumCasas();
                        if (solar.getAsociados().get(j).getTieneHotel())
                            algunoTieneHotel = true;
                    }
                    minNumCasas = maxNumCasas;
                    if (solar.getNumCasas() < minNumCasas)
                            minNumCasas = solar.getNumCasas();
                    for (int j = 0; j < solar.getAsociados().size(); j++)
                        if (solar.getAsociados().get(j).getNumCasas() < minNumCasas)
                            minNumCasas = solar.getAsociados().get(j).getNumCasas();
                    
                    // si son iguales los maximos y minimos, mostramos todos los solares del grupo
                    if (solar.getTieneHotel())
                    {
                        solaresConstruidos.add(solar);
                    } else
                    // si es menor el minimo, solo mostrar los solares del minimo
                    if ((solar.getNumCasas() == maxNumCasas) && (!algunoTieneHotel) && (solar.getNumCasas() > 0))
                    {
                        solaresConstruidos.add(solar);
                    }

            }
        }
        
        return solaresConstruidos;
    }
    
    public void comprarPropiedad(Propiedad propiedad) {
    
        if (!tieneEfectivo(propiedad.getPrecio())) return;
        
        capital -= propiedad.getPrecio();
        propiedad.setPropietario(this);
        propiedades.add(propiedad);
        
    }
    
    public void comprarEnSubasta(Propiedad propiedad, float precio) {
    
        if (!tieneEfectivo(precio)) return;
        
        capital -= precio;
        propiedad.setPropietario(this);
        propiedades.add(propiedad);
        
    }
    
    public void venderPropiedad(Propiedad propiedad, float precio, Jugador comprador) {
    
        propiedad.setPropietario(comprador);
        propiedades.remove(propiedad);
        comprador.getPropiedades().add(propiedad);
        capital += precio;
        comprador.setCapital(comprador.getCapital() - precio);
    
    }
    
    public void venderConstruccion(Solar solar) {
    
        if (!propiedades.contains(solar)) return;
        
        if (solar.getTieneHotel())
        {
            solar.setTieneHotel(false);
            solar.setNumCasas(4);
            capital += solar.getPrecioHotel()/2;

        } else
        if (solar.getNumCasas() > 0)
        {

            solar.setNumCasas(solar.getNumCasas() - 1);
            capital += solar.getPrecioCasa()/2;

        }
    
    }
    
}
